<?php
/*
Plugin Name: Documaps.io Import
Description: Allows site owners to import pages and their content via JSON file upload. It also creates menu structure as specified in the JSON file.
Version: 0.01
Author: Paul Knittel
Author URL: http://www.google.com
*/

function documaps_admin_menu() {
    add_submenu_page( 'tools.php', 'Documaps.io', 'Documaps.io', 'publish_pages', 'documaps', 'documaps_callback' );
}
add_action("admin_menu", 'documaps_admin_menu');

/**
 * Decodes JSON data if JSON file is uploaded.
 *
 * @return null
 */
function documaps_callback() {

    echo '<h2>Import JSON Data</h2>';

    if($_POST['submit'] == 'Import' && $_FILES['documaps_file']['name']) {

        // Check if file size is within the limit
        if(is_file_within_limit()) {
            $file_content = file_get_contents($_FILES['documaps_file']['tmp_name']);
            $array = json_decode($file_content, true);

            if($array !== null ) {

                if($_POST['delete_pages'] == 'yes') {
                    delete_existing_pages();
                }

                delete_existing_menu();
                $menu_id = create_site_menu();

                if($menu_id) {
                    import_jason_data($menu_id, $array);
                    echo '<br><h3>All done. <a href="'.home_url().'" target="_blank">Have fun!</a></h3>';
                } else {
                    echo '<h2>Failed to create site menu!</h2>';
                }

            } else {
                echo '<h2>The uploaded file does not contain data in proper JASON format! <a href="'.site_url('wp-admin/tools.php?page=documaps').'">Try again</a>.</h2>';
            }
        } else {
            echo '<h2>The uploaded file exceeds maximum limit! <a href="'.site_url('wp-admin/tools.php?page=documaps').'">Try again</a></h2></h2>';
        }
    } elseif($_POST['submit'] == 'Import' && !$_FILES['documaps_file']['name']) {
        echo '<h2>Oops, you didn\'t uploaded any file! <a href="'.site_url('wp-admin/tools.php?page=documaps').'">Try again</a></h2>';
    }
    else {
        show_file_upload_form();
    }
}

/**
 * Checks whether uploaded file is within limit.
 *
 * @return boolean
 */
function is_file_within_limit() {
    $file_size = $_FILES['documaps_file']['size'];
    if($file_size > 2000000) {
        return false;
    } else {
        return true;
    }
}

/**
 * Shows the form to upload JSON file.
 *
 * @return null
 */
function show_file_upload_form() {
?>
    <p>This importer allows you to import pages structure from a JSON file into your WordPress website. Pick a JSON file to import and click the Import button.</p>
    <form name="documaps_form" id="documaps_form" action="" method="post" enctype="multipart/form-data" onsubmit="return confirmSubmission(this);">
    <p>Do you want to delete the existing pages from your WordPress site?&nbsp;&nbsp;<input type="checkbox" name="delete_pages" id="delete_pages" value="yes" /> </p>
    <p>Choose a JSON file from your computer: (Maximum size: 2MB)&nbsp;<input type="file" name="documaps_file" id="documaps_file" /> </p>
    <p><input type="submit" name="submit" value="Import" /> </p>
    </form>
    <script type="application/javascript">
        function confirmSubmission() {
            if(this.delete_pages.checked == true) {
                var action = confirm('Do you really want to delete your existing pages?');
                if(action == true) {
                    return true;
                } else {
                    return false;
                }
            }
        }
    </script>
<?php
}

/**
 * Creates empty site menu.
 *
 * @return int $menu_id The ID of the menu created.
 */
function create_site_menu() {
    // Create menu
    $menu_name = 'Documaps.io Site Menu';
    $menu_exists = wp_get_nav_menu_object($menu_name);

    if(!$menu_exists) {
        $menu_id = wp_create_nav_menu($menu_name);
    }

    return $menu_id;
}

/**
 * Import the data and create pages and site menu.
 *
 * @param int $menu_id The main menu ID.
 * @param array $array The array created from JSON data.
 * @param int $parent_id The parent ID of the page being created.
 * @param int $menu_parent_id The parent ID of the menu being created.
 * @return null
 */
function import_jason_data($menu_id, $array, $parent_id = 0, $menu_parent_id = 0) {
    // Create pages as per JSON data
    foreach($array as $key => $val) {

        // Create menu item
        $menu_post_id = wp_update_nav_menu_item($menu_id, 0, array(
                                'menu-item-title' =>  __($val['name']),
                                'menu-item-url' => site_url( sanitize_title($val['name']).'/' ),
                                'menu-item-parent-id' => $menu_parent_id,
                                'menu-item-status' => 'publish'));

        if($val['wpimport'] != 'false' && $val['name'] != '') {

            $post = array(
                'post_content'   => $val['html'],
                'post_title'      => $val['name'],
                'post_status'    => 'publish',
                'post_type'      => 'page',
                'post_parent'    => $parent_id
            );

            $post_id = wp_insert_post($post);
        }

        if($post_id != 0) {
            echo 'Importing page...'.$val['name'].' (ID: '.$post_id.') - Done!'.'<br>';

            if(is_array($val['children']) && count($val['children']) > 0) {
                import_jason_data($menu_id, $val['children'], $post_id, $menu_post_id);
            }
        }
    }
}

/**
 * Delete existing pages.
 *
 * @return null
 */
function delete_existing_pages() {
    // Delete pages
    $pages = get_pages(array('post_type' => 'page'));
    foreach( $pages as $page ) {
        wp_delete_post($page->ID, true);
    }
}

/**
 * Delete existing menu.
 *
 * @return null
 */
function delete_existing_menu() {
    global $wpdb;
    // Delete Documaps menu if exists
    $menu_term = get_term_by('name', 'Documaps.io Site Menu', 'nav_menu');

    if($menu_term !== false) {

        wp_delete_term($menu_term->term_id, 'nav_menu');

        $sql = "SELECT object_id
                FROM ".$wpdb->prefix."term_relationships
                WHERE term_taxonomy_id = ".$menu_term->term_taxonomy_id;

        $rows = $wpdb->get_results($sql);

        if(count($rows) > 0) {
            foreach($rows as $row) {
                wp_delete_post($row->object_id, true);
            }
        }
    }
}